/*
 * Test12.c --  A test of pcthread_setprio()
 */
 
#include <stdlib.h>
#include <stdio.h>
#include "pcthread.h"

#define N 10

static pcthread_t       **child_th, *initial_th;
static pcthread_attr_t  th_attr_handle;

static void null_task( void );
static int    dummy;

main()
{
    int i;
    unsigned original_p, max_p, def_p;
    
    /*
     * Allocate handles for each of the children.  We don't have to disable
     * preemption because PCthreads has not yet been initialized.
     */

    child_th = ( pcthread_t **) malloc( N * sizeof( pcthread_t *));
    for(i = 0; i < N; i++ )
         child_th[i] = (pcthread_t *) malloc( sizeof( pcthread_t ));

    (void) pcthread_attr_create( (pcthread_attr_t *) &th_attr_handle );
    max_p = pcthread_attr_getmaxprio_ns();
    
    /*
     * Elevate the currently active thread's priority to the max allowable
     * priority level.  Create a N threads whose priorities are randomly
     * distributed throughout the priority levels
     * When this loop completes N threads will be resident in the ready queue.
     */
     
    initial_th = pcthread_self();
    original_p = pcthread_setprio( initial_th, max_p );

    for(i = 0; i < N; i++ )
    {
         def_p = rand() % max_p;
         (void) pcthread_attr_setprio( (pcthread_attr_t *) &th_attr_handle,
                                  def_p );    
         (void) pcthread_create(  child_th[i], 
                                  th_attr_handle , 
                                  (pcthread_action_t) null_task,
                                  (pcthread_addr_t *) &dummy );
    }

    /*
     * Now restore the creating thread's original priority.
     */
     
    (void) pcthread_setprio( initial_th, original_p );    

    return 0;
}

void
null_task()
{
    pcthread_t *th_handle;
    int p, ctx;

    th_handle = pcthread_self();
    p = pcthread_getprio( th_handle );        

    ctx = pcthread_disable_preemption_ns();
    printf("Thread[%u], priority %d complete...\n", th_handle->object_id, p );
    pcthread_enable_preemption_ns( ctx );
    
    return;
}