/*
 * Test4d.c -- Create and execute 10 threads of which the first 5 carry
 *             the round robin scheduling attribute while the second five
 *             carry the fcfs_c attribute.
 */
 
#include <stdlib.h>
#include <stdio.h>
#include "pcthread.h"

#define THREADS 10
static pcthread_t       th_handle[THREADS];
static pcthread_attr_t  th_attr;

static void compute( void );

static int    dummy;
static unsigned long bigsum = 0;

main()
{
    int  i;
    
    /*
     * Create a thread attributes object (actually a handle to the object)
     * that, when passed to the pcthread_create() routine will result in
     * the creation of a thread with a 256 word stack and a scheduling
     * discipline of first come first served.
     */
     
    (void) pcthread_attr_create( (pcthread_attr_t *) &th_attr);
    (void) pcthread_attr_setstacksize( (pcthread_attr_t *)&th_attr, 256 );
    (void) pcthread_attr_setsched((pcthread_attr_t *)&th_attr, round_robin_c );
    (void) pcthread_attr_setquantum_ns( (pcthread_attr_t *)&th_attr, 4 );

    /*
     * Create a number of threads, each of whom will contend for the
     * processor.  Note that all are created at the same priority.
     */     

    pcthread_enable_trace_ns();

    for(i = 0; i < THREADS/2; i++ )
         (void) pcthread_create(  (pcthread_t *) &th_handle[i],
                                  th_attr,
                                  (pcthread_action_t) compute,
                                  (pcthread_addr_t *) &dummy );


    (void) pcthread_attr_setsched((pcthread_attr_t *)&th_attr, fcfs_c );
    for(i = THREADS/2; i < THREADS; i++ )
         (void) pcthread_create(  (pcthread_t *) &th_handle[i],
                                  th_attr,
                                  (pcthread_action_t) compute,
                                  (pcthread_addr_t *) &dummy );

 
    return;
}

void
compute()
{
    unsigned ticks = 0, i, j;
    unsigned long sum = 0;    

    for(i = 0; i < 30000; i++ )
         for(j = 0; j < 100; j++ )
         {
              sum += j + i;
              if( sum++ == 80000 )
              {
                   sum = 0;
                   ticks += 1;
              }
         }         

    bigsum = sum;
}